@@ -120,272 +120,6 @@ llvmpipe_get_name(struct pipe_screen *screen)
 }
 
 
-static int
-llvmpipe_get_param(struct pipe_screen *screen, enum pipe_cap param)
-{
-#if defined(HAVE_LIBDRM) && defined(HAVE_LINUX_UDMABUF_H)
-   struct llvmpipe_screen *lscreen = llvmpipe_screen(screen);
-#endif
-
-   switch (param) {
-#if defined(HAVE_LIBDRM) && defined(HAVE_LINUX_UDMABUF_H)
-   case PIPE_CAP_DMABUF:
-      if (lscreen->udmabuf_fd != -1)
-         return DRM_PRIME_CAP_IMPORT | DRM_PRIME_CAP_EXPORT;
-      else
-         return DRM_PRIME_CAP_IMPORT;
-#endif
-   case PIPE_CAP_NPOT_TEXTURES:
-   case PIPE_CAP_MIXED_FRAMEBUFFER_SIZES:
-   case PIPE_CAP_MIXED_COLOR_DEPTH_BITS:
-   case PIPE_CAP_ANISOTROPIC_FILTER:
-      return 1;
-   case PIPE_CAP_FRAGMENT_SHADER_TEXTURE_LOD:
-   case PIPE_CAP_FRAGMENT_SHADER_DERIVATIVES:
-      return 1;
-   case PIPE_CAP_MAX_DUAL_SOURCE_RENDER_TARGETS:
-      return 1;
-   case PIPE_CAP_MAX_STREAM_OUTPUT_BUFFERS:
-      return PIPE_MAX_SO_BUFFERS;
-   case PIPE_CAP_MAX_RENDER_TARGETS:
-      return PIPE_MAX_COLOR_BUFS;
-   case PIPE_CAP_OCCLUSION_QUERY:
-   case PIPE_CAP_QUERY_TIMESTAMP:
-   case PIPE_CAP_TIMER_RESOLUTION:
-   case PIPE_CAP_QUERY_TIME_ELAPSED:
-      return 1;
-   case PIPE_CAP_QUERY_PIPELINE_STATISTICS:
-      return 1;
-   case PIPE_CAP_TEXTURE_MIRROR_CLAMP:
-   case PIPE_CAP_TEXTURE_MIRROR_CLAMP_TO_EDGE:
-      return 1;
-   case PIPE_CAP_TEXTURE_SWIZZLE:
-   case PIPE_CAP_TEXTURE_SHADOW_LOD:
-      return 1;
-   case PIPE_CAP_MAX_TEXTURE_2D_SIZE:
-      return 1 << (LP_MAX_TEXTURE_2D_LEVELS - 1);
-   case PIPE_CAP_MAX_TEXTURE_3D_LEVELS:
-      return LP_MAX_TEXTURE_3D_LEVELS;
-   case PIPE_CAP_MAX_TEXTURE_CUBE_LEVELS:
-      return LP_MAX_TEXTURE_CUBE_LEVELS;
-   case PIPE_CAP_MAX_TEXTURE_ARRAY_LAYERS:
-      return LP_MAX_TEXTURE_ARRAY_LAYERS;
-   case PIPE_CAP_BLEND_EQUATION_SEPARATE:
-      return 1;
-   case PIPE_CAP_INDEP_BLEND_ENABLE:
-      return 1;
-   case PIPE_CAP_INDEP_BLEND_FUNC:
-      return 1;
-   case PIPE_CAP_FS_COORD_ORIGIN_UPPER_LEFT:
-   case PIPE_CAP_FS_COORD_PIXEL_CENTER_INTEGER:
-   case PIPE_CAP_FS_COORD_PIXEL_CENTER_HALF_INTEGER:
-      return 1;
-   case PIPE_CAP_PRIMITIVE_RESTART:
-   case PIPE_CAP_PRIMITIVE_RESTART_FIXED_INDEX:
-      return 1;
-   case PIPE_CAP_DEPTH_CLIP_DISABLE:
-      return 1;
-   case PIPE_CAP_DEPTH_CLAMP_ENABLE:
-      return 1;
-   case PIPE_CAP_SHADER_STENCIL_EXPORT:
-      return 1;
-   case PIPE_CAP_VS_INSTANCEID:
-   case PIPE_CAP_VERTEX_ELEMENT_INSTANCE_DIVISOR:
-   case PIPE_CAP_START_INSTANCE:
-      return 1;
-   case PIPE_CAP_SEAMLESS_CUBE_MAP:
-   case PIPE_CAP_SEAMLESS_CUBE_MAP_PER_TEXTURE:
-      return 1;
-   /* this is a lie could support arbitrary large offsets */
-   case PIPE_CAP_MIN_TEXTURE_GATHER_OFFSET:
-   case PIPE_CAP_MIN_TEXEL_OFFSET:
-      return -32;
-   case PIPE_CAP_MAX_TEXTURE_GATHER_OFFSET:
-   case PIPE_CAP_MAX_TEXEL_OFFSET:
-      return 31;
-   case PIPE_CAP_CONDITIONAL_RENDER:
-   case PIPE_CAP_TEXTURE_BARRIER:
-      return 1;
-   case PIPE_CAP_MAX_STREAM_OUTPUT_SEPARATE_COMPONENTS:
-   case PIPE_CAP_MAX_STREAM_OUTPUT_INTERLEAVED_COMPONENTS:
-      return 16*4;
-   case PIPE_CAP_MAX_GEOMETRY_OUTPUT_VERTICES:
-   case PIPE_CAP_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS:
-      return 1024;
-   case PIPE_CAP_MAX_VERTEX_STREAMS:
-      return 4;
-   case PIPE_CAP_MAX_VERTEX_ATTRIB_STRIDE:
-      return 2048;
-   case PIPE_CAP_STREAM_OUTPUT_PAUSE_RESUME:
-   case PIPE_CAP_STREAM_OUTPUT_INTERLEAVE_BUFFERS:
-   case PIPE_CAP_VERTEX_COLOR_UNCLAMPED:
-   case PIPE_CAP_VERTEX_COLOR_CLAMPED:
-      return 1;
-   case PIPE_CAP_GLSL_FEATURE_LEVEL_COMPATIBILITY:
-   case PIPE_CAP_GLSL_FEATURE_LEVEL:
-      return 450;
-   case PIPE_CAP_COMPUTE:
-      return GALLIVM_COROUTINES;
-   case PIPE_CAP_USER_VERTEX_BUFFERS:
-      return 1;
-   case PIPE_CAP_TGSI_TEXCOORD:
-   case PIPE_CAP_DRAW_INDIRECT:
-      return 1;
-
-   case PIPE_CAP_CUBE_MAP_ARRAY:
-      return 1;
-   case PIPE_CAP_CONSTANT_BUFFER_OFFSET_ALIGNMENT:
-      return 16;
-   case PIPE_CAP_MIN_MAP_BUFFER_ALIGNMENT:
-      return 64;
-   case PIPE_CAP_TEXTURE_BUFFER_OBJECTS:
-      return 1;
-   case PIPE_CAP_LINEAR_IMAGE_PITCH_ALIGNMENT:
-      return 1;
-   case PIPE_CAP_LINEAR_IMAGE_BASE_ADDRESS_ALIGNMENT:
-      return 1;
-   /* Adressing that many 64bpp texels fits in an i32 so this is a reasonable value */
-   case PIPE_CAP_MAX_TEXEL_BUFFER_ELEMENTS_UINT:
-      return LP_MAX_TEXEL_BUFFER_ELEMENTS;
-   case PIPE_CAP_TEXTURE_BUFFER_OFFSET_ALIGNMENT:
-      return 16;
-   case PIPE_CAP_TEXTURE_TRANSFER_MODES:
-      return 0;
-   case PIPE_CAP_MAX_VIEWPORTS:
-      return PIPE_MAX_VIEWPORTS;
-   case PIPE_CAP_ENDIANNESS:
-      return PIPE_ENDIAN_NATIVE;
-   case PIPE_CAP_TES_LAYER_VIEWPORT:
-   case PIPE_CAP_VS_LAYER_VIEWPORT:
-      return 1;
-   case PIPE_CAP_MAX_TEXTURE_GATHER_COMPONENTS:
-      return 4;
-   case PIPE_CAP_VS_WINDOW_SPACE_POSITION:
-      return 1;
-   case PIPE_CAP_FS_FINE_DERIVATIVE:
-      return 1;
-   case PIPE_CAP_TGSI_TEX_TXF_LZ:
-   case PIPE_CAP_SAMPLER_VIEW_TARGET:
-      return 1;
-   case PIPE_CAP_FAKE_SW_MSAA:
-      return 0;
-   case PIPE_CAP_TEXTURE_QUERY_LOD:
-   case PIPE_CAP_CONDITIONAL_RENDER_INVERTED:
-   case PIPE_CAP_SHADER_ARRAY_COMPONENTS:
-   case PIPE_CAP_DOUBLES:
-   case PIPE_CAP_INT64:
-   case PIPE_CAP_QUERY_SO_OVERFLOW:
-   case PIPE_CAP_TGSI_DIV:
-      return 1;
-   case PIPE_CAP_VENDOR_ID:
-      return 0xFFFFFFFF;
-   case PIPE_CAP_DEVICE_ID:
-      return 0xFFFFFFFF;
-   case PIPE_CAP_ACCELERATED:
-      return 0;
-   case PIPE_CAP_VIDEO_MEMORY: {
-      /* XXX: Do we want to return the full amount fo system memory ? */
-      uint64_t system_memory;
-
-      if (!os_get_total_physical_memory(&system_memory))
-         return 0;
-
-      if (sizeof(void *) == 4)
-         /* Cap to 2 GB on 32 bits system. We do this because llvmpipe does
-          * eat application memory, which is quite limited on 32 bits. App
-          * shouldn't expect too much available memory. */
-         system_memory = MIN2(system_memory, 2048 << 20);
-
-      return (int)(system_memory >> 20);
-   }
-   case PIPE_CAP_UMA:
-      return 1;
-   case PIPE_CAP_QUERY_MEMORY_INFO:
-      return 1;
-   case PIPE_CAP_CLIP_HALFZ:
-      return 1;
-   case PIPE_CAP_POLYGON_OFFSET_CLAMP:
-   case PIPE_CAP_TEXTURE_FLOAT_LINEAR:
-   case PIPE_CAP_TEXTURE_HALF_FLOAT_LINEAR:
-      return 1;
-   case PIPE_CAP_CULL_DISTANCE:
-      return 1;
-   case PIPE_CAP_COPY_BETWEEN_COMPRESSED_AND_PLAIN_FORMATS:
-      return 1;
-   case PIPE_CAP_MAX_VARYINGS:
-      return 32;
-   case PIPE_CAP_SHADER_BUFFER_OFFSET_ALIGNMENT:
-      return 16;
-   case PIPE_CAP_QUERY_BUFFER_OBJECT:
-      return 1;
-   case PIPE_CAP_DRAW_PARAMETERS:
-      return 1;
-   case PIPE_CAP_FBFETCH:
-      return 8;
-   case PIPE_CAP_FBFETCH_COHERENT:
-   case PIPE_CAP_FBFETCH_ZS:
-   case PIPE_CAP_MULTI_DRAW_INDIRECT:
-   case PIPE_CAP_MULTI_DRAW_INDIRECT_PARAMS:
-      return 1;
-   case PIPE_CAP_DEVICE_RESET_STATUS_QUERY:
-   case PIPE_CAP_ROBUST_BUFFER_ACCESS_BEHAVIOR:
-      return 1;
-   case PIPE_CAP_MAX_SHADER_PATCH_VARYINGS:
-      return 32;
-   case PIPE_CAP_RASTERIZER_SUBPIXEL_BITS:
-      return 8;
-   case PIPE_CAP_PCI_GROUP:
-   case PIPE_CAP_PCI_BUS:
-   case PIPE_CAP_PCI_DEVICE:
-   case PIPE_CAP_PCI_FUNCTION:
-   case PIPE_CAP_ALLOW_MAPPED_BUFFERS_DURING_EXECUTION:
-      return 0;
-
-   case PIPE_CAP_SHAREABLE_SHADERS:
-      /* Can't expose shareable shaders because the draw shaders reference the
-       * draw module's state, which is per-context.
-       */
-      return 0;
-   case PIPE_CAP_MAX_GS_INVOCATIONS:
-      return 32;
-   case PIPE_CAP_MAX_SHADER_BUFFER_SIZE_UINT:
-      return LP_MAX_TGSI_SHADER_BUFFER_SIZE;
-   case PIPE_CAP_FRAMEBUFFER_NO_ATTACHMENT:
-   case PIPE_CAP_TGSI_TG4_COMPONENT_IN_SWIZZLE:
-   case PIPE_CAP_FS_FACE_IS_INTEGER_SYSVAL:
-   case PIPE_CAP_RESOURCE_FROM_USER_MEMORY:
-   case PIPE_CAP_IMAGE_STORE_FORMATTED:
-   case PIPE_CAP_IMAGE_LOAD_FORMATTED:
-      return 1;
-#ifdef PIPE_MEMORY_FD
-   case PIPE_CAP_MEMOBJ:
-      return 1;
-#endif
-   case PIPE_CAP_SAMPLER_REDUCTION_MINMAX:
-   case PIPE_CAP_TEXTURE_QUERY_SAMPLES:
-   case PIPE_CAP_SHADER_GROUP_VOTE:
-   case PIPE_CAP_SHADER_BALLOT:
-   case PIPE_CAP_IMAGE_ATOMIC_FLOAT_ADD:
-   case PIPE_CAP_LOAD_CONSTBUF:
-   case PIPE_CAP_TEXTURE_MULTISAMPLE:
-   case PIPE_CAP_SAMPLE_SHADING:
-   case PIPE_CAP_GL_SPIRV:
-   case PIPE_CAP_POST_DEPTH_COVERAGE:
-   case PIPE_CAP_SHADER_CLOCK:
-   case PIPE_CAP_PACKED_UNIFORMS:
-      return 1;
-   case PIPE_CAP_SYSTEM_SVM:
-      return 1;
-   case PIPE_CAP_ATOMIC_FLOAT_MINMAX:
-      return LLVM_VERSION_MAJOR >= 15;
-   case PIPE_CAP_NIR_IMAGES_AS_DEREF:
-      return 0;
-   default:
-      return u_pipe_screen_get_param_defaults(screen, param);
-   }
-}
-
-
 static int
 llvmpipe_get_shader_param(struct pipe_screen *screen,
                           enum pipe_shader_type shader,
@@ -396,8 +130,7 @@ llvmpipe_get_shader_param(struct pipe_screen *screen,
    case PIPE_SHADER_COMPUTE:
       if ((lscreen->allow_cl) && param == PIPE_SHADER_CAP_SUPPORTED_IRS)
          return ((1 << PIPE_SHADER_IR_TGSI) |
-                 (1 << PIPE_SHADER_IR_NIR) |
-                 (1 << PIPE_SHADER_IR_NIR_SERIALIZED));
+                 (1 << PIPE_SHADER_IR_NIR));
       FALLTHROUGH;
    case PIPE_SHADER_MESH:
    case PIPE_SHADER_TASK:
@@ -440,43 +173,6 @@ llvmpipe_get_shader_param(struct pipe_screen *screen,
 }
 
 
-static float
-llvmpipe_get_paramf(struct pipe_screen *screen, enum pipe_capf param)
-{
-   switch (param) {
-   case PIPE_CAPF_MIN_LINE_WIDTH:
-   case PIPE_CAPF_MIN_LINE_WIDTH_AA:
-   case PIPE_CAPF_MIN_POINT_SIZE:
-   case PIPE_CAPF_MIN_POINT_SIZE_AA:
-      return 1.0;
-   case PIPE_CAPF_POINT_SIZE_GRANULARITY:
-   case PIPE_CAPF_LINE_WIDTH_GRANULARITY:
-      return 0.1;
-   case PIPE_CAPF_MAX_LINE_WIDTH:
-      FALLTHROUGH;
-   case PIPE_CAPF_MAX_LINE_WIDTH_AA:
-      return 255.0; /* arbitrary */
-   case PIPE_CAPF_MAX_POINT_SIZE:
-      FALLTHROUGH;
-   case PIPE_CAPF_MAX_POINT_SIZE_AA:
-      return LP_MAX_POINT_WIDTH; /* arbitrary */
-   case PIPE_CAPF_MAX_TEXTURE_ANISOTROPY:
-      return 16.0; /* not actually signficant at this time */
-   case PIPE_CAPF_MAX_TEXTURE_LOD_BIAS:
-      return 16.0; /* arbitrary */
-   case PIPE_CAPF_MIN_CONSERVATIVE_RASTER_DILATE:
-      return 0.0;
-   case PIPE_CAPF_MAX_CONSERVATIVE_RASTER_DILATE:
-      return 0.0;
-   case PIPE_CAPF_CONSERVATIVE_RASTER_DILATE_GRANULARITY:
-      return 0.0;
-   }
-   /* should only get here on unhandled cases */
-   debug_printf("Unexpected PIPE_CAP %d query\n", param);
-   return 0.0;
-}
-
-
 static int
 llvmpipe_get_compute_param(struct pipe_screen *_screen,
                            enum pipe_shader_ir ir_type,
@@ -587,6 +283,214 @@ llvmpipe_get_compute_param(struct pipe_screen *_screen,
 }
 
 
+static void
+llvmpipe_init_screen_caps(struct pipe_screen *screen)
+{
+   struct pipe_caps *caps = (struct pipe_caps *)&screen->caps;
+
+   u_init_pipe_screen_caps(screen, 0);
+
+#ifdef HAVE_LIBDRM
+   struct llvmpipe_screen *lscreen = llvmpipe_screen(screen);
+#endif
+
+#ifdef HAVE_LIBDRM
+   if (lscreen->winsys->get_fd)
+      caps->dmabuf = DRM_PRIME_CAP_IMPORT | DRM_PRIME_CAP_EXPORT;
+#ifdef HAVE_LINUX_UDMABUF_H
+   else if (lscreen->udmabuf_fd != -1)
+      caps->dmabuf = DRM_PRIME_CAP_IMPORT | DRM_PRIME_CAP_EXPORT;
+   else
+      caps->dmabuf = DRM_PRIME_CAP_IMPORT;
+#endif
+#else
+   caps->dmabuf = 0;
+#endif
+
+#if defined(HAVE_LIBDRM) && defined(HAVE_LINUX_UDMABUF_H)
+   caps->native_fence_fd = lscreen->dummy_sync_fd != -1;
+#endif
+   caps->npot_textures = true;
+   caps->mixed_framebuffer_sizes = true;
+   caps->mixed_color_depth_bits = true;
+   caps->anisotropic_filter = true;
+   caps->fragment_shader_texture_lod = true;
+   caps->fragment_shader_derivatives = true;
+   caps->multiview = 2;
+   caps->max_dual_source_render_targets = 1;
+   caps->max_stream_output_buffers = PIPE_MAX_SO_BUFFERS;
+   caps->max_render_targets = PIPE_MAX_COLOR_BUFS;
+   caps->occlusion_query = true;
+   caps->query_timestamp = true;
+   caps->timer_resolution = true;
+   caps->query_time_elapsed = true;
+   caps->query_pipeline_statistics = true;
+   caps->texture_mirror_clamp = true;
+   caps->texture_mirror_clamp_to_edge = true;
+   caps->texture_swizzle = true;
+   caps->texture_shadow_lod = true;
+   caps->max_texture_2d_size = 1 << (LP_MAX_TEXTURE_2D_LEVELS - 1);
+   caps->max_texture_3d_levels = LP_MAX_TEXTURE_3D_LEVELS;
+   caps->max_texture_cube_levels = LP_MAX_TEXTURE_CUBE_LEVELS;
+   caps->max_texture_array_layers = LP_MAX_TEXTURE_ARRAY_LAYERS;
+   caps->blend_equation_separate = true;
+   caps->indep_blend_enable = true;
+   caps->indep_blend_func = true;
+   caps->fs_coord_origin_upper_left = true;
+   caps->fs_coord_pixel_center_integer = true;
+   caps->fs_coord_pixel_center_half_integer = true;
+   caps->primitive_restart = true;
+   caps->primitive_restart_fixed_index = true;
+   caps->depth_clip_disable = true;
+   caps->depth_clamp_enable = true;
+   caps->shader_stencil_export = true;
+   caps->vs_instanceid = true;
+   caps->vertex_element_instance_divisor = true;
+   caps->start_instance = true;
+   caps->seamless_cube_map = true;
+   caps->seamless_cube_map_per_texture = true;
+   /* this is a lie could support arbitrary large offsets */
+   caps->min_texture_gather_offset =
+   caps->min_texel_offset = -32;
+   caps->max_texture_gather_offset =
+   caps->max_texel_offset = 31;
+   caps->conditional_render = true;
+   caps->texture_barrier = true;
+   caps->max_stream_output_separate_components =
+   caps->max_stream_output_interleaved_components = 16*4;
+   caps->max_geometry_output_vertices =
+   caps->max_geometry_total_output_components = 1024;
+   caps->max_vertex_streams = 4;
+   caps->max_vertex_attrib_stride = 2048;
+   caps->stream_output_pause_resume = true;
+   caps->stream_output_interleave_buffers = true;
+   caps->vertex_color_unclamped = true;
+   caps->vertex_color_clamped = true;
+   caps->glsl_feature_level_compatibility =
+   caps->glsl_feature_level = 450;
+   caps->compute = GALLIVM_COROUTINES;
+   caps->user_vertex_buffers = true;
+   caps->tgsi_texcoord = true;
+   caps->draw_indirect = true;
+
+   caps->cube_map_array = true;
+   caps->constant_buffer_offset_alignment = 16;
+   caps->min_map_buffer_alignment = 64;
+   caps->texture_buffer_objects = true;
+   caps->linear_image_pitch_alignment = 1;
+   caps->linear_image_base_address_alignment = 1;
+   /* Adressing that many 64bpp texels fits in an i32 so this is a reasonable value */
+   caps->max_texel_buffer_elements = LP_MAX_TEXEL_BUFFER_ELEMENTS;
+   caps->texture_buffer_offset_alignment = 16;
+   caps->texture_transfer_modes = 0;
+   caps->max_viewports = PIPE_MAX_VIEWPORTS;
+   caps->endianness = PIPE_ENDIAN_NATIVE;
+   caps->tes_layer_viewport = true;
+   caps->vs_layer_viewport = true;
+   caps->max_texture_gather_components = 4;
+   caps->vs_window_space_position = true;
+   caps->fs_fine_derivative = true;
+   caps->tgsi_tex_txf_lz = true;
+   caps->sampler_view_target = true;
+   caps->fake_sw_msaa = false;
+   caps->texture_query_lod = true;
+   caps->conditional_render_inverted = true;
+   caps->shader_array_components = true;
+   caps->doubles = true;
+   caps->int64 = true;
+   caps->query_so_overflow = true;
+   caps->tgsi_div = true;
+   caps->vendor_id = 0xFFFFFFFF;
+   caps->device_id = 0xFFFFFFFF;
+
+   /* XXX: Do we want to return the full amount fo system memory ? */
+   uint64_t system_memory;
+   if (os_get_total_physical_memory(&system_memory)) {
+      if (sizeof(void *) == 4)
+         /* Cap to 2 GB on 32 bits system. We do this because llvmpipe does
+          * eat application memory, which is quite limited on 32 bits. App
+          * shouldn't expect too much available memory. */
+         system_memory = MIN2(system_memory, 2048 << 20);
+
+      caps->video_memory = system_memory >> 20;
+   } else {
+      caps->video_memory = 0;
+   }
+
+   caps->uma = true;
+   caps->query_memory_info = true;
+   caps->clip_halfz = true;
+   caps->polygon_offset_clamp = true;
+   caps->texture_float_linear = true;
+   caps->texture_half_float_linear = true;
+   caps->cull_distance = true;
+   caps->copy_between_compressed_and_plain_formats = true;
+   caps->max_varyings = 32;
+   caps->shader_buffer_offset_alignment = 16;
+   caps->query_buffer_object = true;
+   caps->draw_parameters = true;
+   caps->fbfetch = 8;
+   caps->fbfetch_coherent = true;
+   caps->fbfetch_zs = true;
+   caps->multi_draw_indirect = true;
+   caps->multi_draw_indirect_params = true;
+   caps->device_reset_status_query = true;
+   caps->robust_buffer_access_behavior = true;
+   caps->max_shader_patch_varyings = 32;
+   caps->rasterizer_subpixel_bits = 8;
+   caps->pci_group =
+   caps->pci_bus =
+   caps->pci_device =
+   caps->pci_function = 0;
+   caps->allow_mapped_buffers_during_execution = false;
+
+   /* Can't expose shareable shaders because the draw shaders reference the
+    * draw module's state, which is per-context.
+    */
+   caps->shareable_shaders = false;
+   caps->max_gs_invocations = 32;
+   caps->max_shader_buffer_size = LP_MAX_TGSI_SHADER_BUFFER_SIZE;
+   caps->framebuffer_no_attachment = true;
+   caps->tgsi_tg4_component_in_swizzle = true;
+   caps->fs_face_is_integer_sysval = true;
+   caps->resource_from_user_memory = true;
+   caps->image_store_formatted = true;
+   caps->image_load_formatted = true;
+#ifdef PIPE_MEMORY_FD
+   caps->memobj = true;
+#endif
+   caps->sampler_reduction_minmax = true;
+   caps->texture_query_samples = true;
+   caps->shader_group_vote = true;
+   caps->shader_ballot = true;
+   caps->image_atomic_float_add = true;
+   caps->load_constbuf = true;
+   caps->texture_multisample = true;
+   caps->sample_shading = true;
+   caps->gl_spirv = true;
+   caps->post_depth_coverage = true;
+   caps->shader_clock = true;
+   caps->packed_uniforms = true;
+   caps->system_svm = true;
+   caps->atomic_float_minmax = LLVM_VERSION_MAJOR >= 15;
+   caps->nir_images_as_deref = false;
+   caps->alpha_to_coverage_dither_control = true;
+
+   caps->min_line_width =
+   caps->min_line_width_aa =
+   caps->min_point_size =
+   caps->min_point_size_aa = 1.0;
+   caps->point_size_granularity =
+   caps->line_width_granularity = 0.1;
+   caps->max_line_width =
+   caps->max_line_width_aa = 255.0; /* arbitrary */
+   caps->max_point_size =
+   caps->max_point_size_aa = LP_MAX_POINT_WIDTH; /* arbitrary */
+   caps->max_texture_anisotropy = 16.0; /* not actually signficant at this time */
+   caps->max_texture_lod_bias = 16.0; /* arbitrary */
+}
+
+
 static void
 llvmpipe_get_driver_uuid(struct pipe_screen *pscreen, char *uuid)
 {
@@ -635,6 +539,7 @@ static const struct nir_shader_compiler_options gallivm_nir_options = {
    .lower_pack_unorm_2x16 = true,
    .lower_pack_unorm_4x8 = true,
    .lower_pack_half_2x16 = true,
+   .lower_pack_64_4x16 = true,
    .lower_pack_split = true,
    .lower_unpack_snorm_2x16 = true,
    .lower_unpack_snorm_4x8 = true,
@@ -652,7 +557,6 @@ static const struct nir_shader_compiler_options gallivm_nir_options = {
    .lower_int64_options = nir_lower_imul_2x32_64,
    .lower_doubles_options = nir_lower_dround_even,
    .max_unroll_iterations = 32,
-   .use_interpolated_input_intrinsics = true,
    .lower_to_scalar = true,
    .lower_uniforms_to_ubo = true,
    .lower_vector_cmp = true,
@@ -660,15 +564,18 @@ static const struct nir_shader_compiler_options gallivm_nir_options = {
    .support_16bit_alu = true,
    .lower_fisnormal = true,
    .lower_fquantize2f16 = true,
+   .lower_fminmax_signed_zero = true,
    .driver_functions = true,
+   .scalarize_ddx = true,
+   .support_indirect_inputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
+   .support_indirect_outputs = (uint8_t)BITFIELD_MASK(PIPE_SHADER_TYPES),
 };
 
 
 static char *
 llvmpipe_finalize_nir(struct pipe_screen *screen,
-                      void *nirptr)
+                      struct nir_shader *nir)
 {
-   struct nir_shader *nir = (struct nir_shader *)nirptr;
    lp_build_opt_nir(nir);
    return NULL;
 }
@@ -1143,10 +1050,8 @@ llvmpipe_create_screen(struct sw_winsys *winsys)
    screen->base.get_vendor = llvmpipe_get_vendor;
    screen->base.get_device_vendor = llvmpipe_get_vendor; // TODO should be the CPU vendor
    screen->base.get_screen_fd = llvmpipe_screen_get_fd;
-   screen->base.get_param = llvmpipe_get_param;
    screen->base.get_shader_param = llvmpipe_get_shader_param;
    screen->base.get_compute_param = llvmpipe_get_compute_param;
-   screen->base.get_paramf = llvmpipe_get_paramf;
    screen->base.get_compiler_options = llvmpipe_get_compiler_options;
    screen->base.is_format_supported = llvmpipe_is_format_supported;
 
@@ -1176,6 +1081,7 @@ llvmpipe_create_screen(struct sw_winsys *winsys)
 
 #if defined(HAVE_LIBDRM) && defined(HAVE_LINUX_UDMABUF_H)
    screen->udmabuf_fd = open("/dev/udmabuf", O_RDWR);
+   llvmpipe_init_screen_fence_funcs(&screen->base);
 #endif
 
    uint64_t alignment;
@@ -1190,6 +1096,8 @@ llvmpipe_create_screen(struct sw_winsys *winsys)
    screen->fd_mem_alloc = os_create_anonymous_file(0, "allocation fd");
 #endif
 
+   llvmpipe_init_screen_caps(&screen->base);
+
    snprintf(screen->renderer_string, sizeof(screen->renderer_string),
             "llvmpipe (LLVM " MESA_LLVM_VERSION_STRING ", %u bits)",
             lp_build_init_native_width() );
