@@ -1,27 +1,7 @@
 /* -*- mesa-c++  -*-
- *
- * Copyright (c) 2022 Collabora LTD
- *
+ * Copyright 2022 Collabora LTD
  * Author: Gert Wollny <gert.wollny@collabora.com>
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * on the rights to use, copy, modify, merge, publish, distribute, sub
- * license, and/or sell copies of the Software, and to permit persons to whom
- * the Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
- * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
- * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
- * USE OR OTHER DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  */
 
 #include "sfn_shader.h"
@@ -39,6 +19,7 @@
 #include "sfn_instr_fetch.h"
 #include "sfn_instr_lds.h"
 #include "sfn_instr_mem.h"
+#include "sfn_instr_tex.h"
 #include "sfn_liverangeevaluator.h"
 #include "sfn_shader_cs.h"
 #include "sfn_shader_fs.h"
@@ -848,6 +829,46 @@ Shader::process_instr(nir_instr *instr)
    return m_instr_factory->from_nir(instr, *this);
 }
 
+bool
+Shader::emit_tex_fdd(const nir_intrinsic_instr* intr, int opcode, bool fine)
+{
+   auto& value_factory_ = value_factory();
+
+   int ncomp = intr->def.num_components;
+   RegisterVec4::Swizzle src_swz = {7, 7, 7, 7};
+   RegisterVec4::Swizzle tmp_swz = {7, 7, 7, 7};
+   for (auto i = 0; i < ncomp; ++i) {
+      src_swz[i] = i;
+      tmp_swz[i] = i;
+   }
+
+   auto src = value_factory_.src_vec4(intr->src[0], pin_none, src_swz);
+
+   auto tmp = value_factory_.temp_vec4(pin_group, tmp_swz);
+   AluInstr *mv = nullptr;
+   for (int i = 0; i < ncomp; ++i) {
+      mv = new AluInstr(op1_mov, tmp[i], src[i], AluInstr::write);
+      emit_instruction(mv);
+   }
+   if (mv)
+      mv->set_alu_flag(alu_last_instr);
+
+   auto dst = value_factory_.dest_vec4(intr->def, pin_group);
+   RegisterVec4::Swizzle dst_swz = {7, 7, 7, 7};
+   for (auto i = 0; i < ncomp; ++i) {
+      dst_swz[i] = i;
+   }
+
+   auto tex = new TexInstr((TexInstr::Opcode)opcode, dst, dst_swz, tmp, R600_MAX_CONST_BUFFERS, nullptr);
+
+   if (fine)
+      tex->set_tex_flag(TexInstr::grad_fine);
+
+   emit_instruction(tex);
+
+   return true;
+}
+
 bool
 Shader::process_intrinsic(nir_intrinsic_instr *intr)
 {
@@ -891,6 +912,16 @@ Shader::process_intrinsic(nir_intrinsic_instr *intr)
       return emit_atomic_local_shared(intr);
    case nir_intrinsic_shader_clock:
       return emit_shader_clock(intr);
+   case nir_intrinsic_ddx:
+   case nir_intrinsic_ddx_coarse:
+      return emit_tex_fdd(intr, TexInstr::get_gradient_h, false);
+   case nir_intrinsic_ddx_fine:
+      return emit_tex_fdd(intr, TexInstr::get_gradient_h, true);
+   case nir_intrinsic_ddy:
+   case nir_intrinsic_ddy_coarse:
+      return emit_tex_fdd(intr, TexInstr::get_gradient_v, false);
+   case nir_intrinsic_ddy_fine:
+      return emit_tex_fdd(intr, TexInstr::get_gradient_v, true);
    case nir_intrinsic_load_reg:
       return emit_load_reg(intr);
    case nir_intrinsic_load_reg_indirect:
