@@ -121,13 +121,6 @@ zink_reset_batch_state(struct zink_context *ctx, struct zink_batch_state *bs)
       VKSCR(DestroyQueryPool)(screen->dev, *pool, NULL);
    util_dynarray_clear(&bs->dead_querypools);
 
-   util_dynarray_foreach(&bs->dgc.pipelines, VkPipeline, pipeline)
-      VKSCR(DestroyPipeline)(screen->dev, *pipeline, NULL);
-   util_dynarray_clear(&bs->dgc.pipelines);
-   util_dynarray_foreach(&bs->dgc.layouts, VkIndirectCommandsLayoutNV, iclayout)
-      VKSCR(DestroyIndirectCommandsLayoutNV)(screen->dev, *iclayout, NULL);
-   util_dynarray_clear(&bs->dgc.layouts);
-
    /* samplers are appended to the batch state in which they are destroyed
     * to ensure deferred deletion without destroying in-use objects
     */
@@ -138,10 +131,10 @@ zink_reset_batch_state(struct zink_context *ctx, struct zink_batch_state *bs)
 
    zink_batch_descriptor_reset(screen, bs);
 
-   util_dynarray_foreach(&bs->freed_sparse_backing_bos, struct zink_bo, bo) {
+   while (util_dynarray_contains(&bs->freed_sparse_backing_bos, struct zink_bo*)) {
+      struct zink_bo *bo = util_dynarray_pop(&bs->freed_sparse_backing_bos, struct zink_bo*);
       zink_bo_unref(screen, bo);
    }
-   util_dynarray_clear(&bs->freed_sparse_backing_bos);
 
    /* programs are refcounted and batch-tracked */
    set_foreach_remove(&bs->programs, entry) {
@@ -152,16 +145,18 @@ zink_reset_batch_state(struct zink_context *ctx, struct zink_batch_state *bs)
 
    bs->resource_size = 0;
    bs->signal_semaphore = VK_NULL_HANDLE;
+   bs->sparse_semaphore = VK_NULL_HANDLE;
    util_dynarray_clear(&bs->wait_semaphore_stages);
+   util_dynarray_clear(&bs->wait_semaphores);
 
    bs->present = VK_NULL_HANDLE;
    /* check the arrays first to avoid locking unnecessarily */
-   if (util_dynarray_contains(&bs->acquires, VkSemaphore) || util_dynarray_contains(&bs->wait_semaphores, VkSemaphore)) {
+   if (util_dynarray_contains(&bs->acquires, VkSemaphore) || util_dynarray_contains(&bs->tracked_semaphores, VkSemaphore)) {
       simple_mtx_lock(&screen->semaphores_lock);
       util_dynarray_append_dynarray(&screen->semaphores, &bs->acquires);
       util_dynarray_clear(&bs->acquires);
-      util_dynarray_append_dynarray(&screen->semaphores, &bs->wait_semaphores);
-      util_dynarray_clear(&bs->wait_semaphores);
+      util_dynarray_append_dynarray(&screen->semaphores, &bs->tracked_semaphores);
+      util_dynarray_clear(&bs->tracked_semaphores);
       simple_mtx_unlock(&screen->semaphores_lock);
    }
    if (util_dynarray_contains(&bs->signal_semaphores, VkSemaphore) || util_dynarray_contains(&bs->fd_wait_semaphores, VkSemaphore)) {
@@ -178,8 +173,8 @@ zink_reset_batch_state(struct zink_context *ctx, struct zink_batch_state *bs)
       zink_fence_reference(screen, mfence, NULL);
    util_dynarray_clear(&bs->fences);
 
-   bs->unordered_write_access = 0;
-   bs->unordered_write_stages = 0;
+   bs->unordered_write_access = VK_ACCESS_NONE;
+   bs->unordered_write_stages = VK_PIPELINE_STAGE_NONE;
 
    /* only increment batch generation if previously in-use to avoid false detection of batch completion */
    if (bs->fence.submitted)
@@ -188,14 +183,16 @@ zink_reset_batch_state(struct zink_context *ctx, struct zink_batch_state *bs)
     * before the state is reused
     */
    bs->fence.submitted = false;
-   bs->has_barriers = false;
-   bs->has_unsync = false;
    if (bs->fence.batch_id)
       zink_screen_update_last_finished(screen, bs->fence.batch_id);
    bs->fence.batch_id = 0;
    bs->usage.usage = 0;
    bs->next = NULL;
    bs->last_added_obj = NULL;
+
+   bs->has_work = false;
+   bs->has_reordered_work = false;
+   bs->has_unsync = false;
 }
 
 /* this is where deferred resource unrefs occur */
@@ -251,8 +248,8 @@ pop_batch_state(struct zink_context *ctx)
    const struct zink_batch_state *bs = ctx->batch_states;
    ctx->batch_states = bs->next;
    ctx->batch_states_count--;
-   if (ctx->last_fence == &bs->fence)
-      ctx->last_fence = NULL;
+   if (ctx->last_batch_state == bs)
+      ctx->last_batch_state = NULL;
 }
 
 /* reset all batch states and append to the free state list
@@ -301,8 +298,6 @@ zink_batch_state_destroy(struct zink_screen *screen, struct zink_batch_state *bs
    free(bs->sparse_objs.objs);
    util_dynarray_fini(&bs->freed_sparse_backing_bos);
    util_dynarray_fini(&bs->dead_querypools);
-   util_dynarray_fini(&bs->dgc.pipelines);
-   util_dynarray_fini(&bs->dgc.layouts);
    util_dynarray_fini(&bs->swapchain_obj);
    util_dynarray_fini(&bs->zombie_samplers);
    util_dynarray_fini(&bs->unref_resources);
@@ -314,6 +309,7 @@ zink_batch_state_destroy(struct zink_screen *screen, struct zink_batch_state *bs
    util_dynarray_fini(&bs->wait_semaphore_stages);
    util_dynarray_fini(&bs->fd_wait_semaphores);
    util_dynarray_fini(&bs->fd_wait_semaphore_stages);
+   util_dynarray_fini(&bs->tracked_semaphores);
    util_dynarray_fini(&bs->acquire_flags);
    unsigned num_mfences = util_dynarray_num_elements(&bs->fence.mfences, void *);
    struct zink_tc_fence **mfence = bs->fence.mfences.data;
@@ -393,11 +389,10 @@ create_batch_state(struct zink_context *ctx)
    SET_CREATE_OR_FAIL(&bs->dmabuf_exports);
    util_dynarray_init(&bs->signal_semaphores, NULL);
    util_dynarray_init(&bs->wait_semaphores, NULL);
+   util_dynarray_init(&bs->tracked_semaphores, NULL);
    util_dynarray_init(&bs->fd_wait_semaphores, NULL);
    util_dynarray_init(&bs->fences, NULL);
    util_dynarray_init(&bs->dead_querypools, NULL);
-   util_dynarray_init(&bs->dgc.pipelines, NULL);
-   util_dynarray_init(&bs->dgc.layouts, NULL);
    util_dynarray_init(&bs->wait_semaphore_stages, NULL);
    util_dynarray_init(&bs->fd_wait_semaphore_stages, NULL);
    util_dynarray_init(&bs->zombie_samplers, NULL);
@@ -412,6 +407,7 @@ create_batch_state(struct zink_context *ctx)
 
    cnd_init(&bs->usage.flush);
    mtx_init(&bs->usage.mtx, mtx_plain);
+   simple_mtx_init(&bs->ref_lock, mtx_plain);
    simple_mtx_init(&bs->exportable_lock, mtx_plain);
    memset(&bs->buffer_indices_hashlist, -1, sizeof(bs->buffer_indices_hashlist));
 
@@ -439,7 +435,7 @@ find_unused_state(struct zink_batch_state *bs)
 
 /* find a "free" batch state */
 static struct zink_batch_state *
-get_batch_state(struct zink_context *ctx, struct zink_batch *batch)
+get_batch_state(struct zink_context *ctx)
 {
    struct zink_screen *screen = zink_screen(ctx->base.screen);
    struct zink_batch_state *bs = NULL;
@@ -477,7 +473,7 @@ get_batch_state(struct zink_context *ctx, struct zink_batch *batch)
    if (bs) {
       zink_reset_batch_state(ctx, bs);
    } else {
-      if (!batch->state) {
+      if (!ctx->bs) {
          /* this is batch init, so create a few more states for later use */
          for (int i = 0; i < 3; i++) {
             struct zink_batch_state *state = create_batch_state(ctx);
@@ -494,26 +490,24 @@ get_batch_state(struct zink_context *ctx, struct zink_batch *batch)
    return bs;
 }
 
-/* reset the batch object: get a new state and unset 'has_work' to disable flushing */
+/* reset the batch object: get a new state and unset 'state->has_work' to disable flushing */
 void
-zink_reset_batch(struct zink_context *ctx, struct zink_batch *batch)
+zink_reset_batch(struct zink_context *ctx)
 {
-   batch->state = get_batch_state(ctx, batch);
-   assert(batch->state);
-
-   batch->has_work = false;
+   ctx->bs = get_batch_state(ctx);
+   assert(ctx->bs);
 }
 
 void
 zink_batch_bind_db(struct zink_context *ctx)
 {
    struct zink_screen *screen = zink_screen(ctx->base.screen);
-   struct zink_batch *batch = &ctx->batch;
+   struct zink_batch_state *bs = ctx->bs;
    unsigned count = 1;
    VkDescriptorBufferBindingInfoEXT infos[2] = {0};
    infos[0].sType = VK_STRUCTURE_TYPE_DESCRIPTOR_BUFFER_BINDING_INFO_EXT;
-   infos[0].address = batch->state->dd.db->obj->bda;
-   infos[0].usage = batch->state->dd.db->obj->vkusage;
+   infos[0].address = bs->dd.db->obj->bda;
+   infos[0].usage = bs->dd.db->obj->vkusage;
    assert(infos[0].usage);
 
    if (ctx->dd.bindless_init) {
@@ -523,19 +517,20 @@ zink_batch_bind_db(struct zink_context *ctx)
       assert(infos[1].usage);
       count++;
    }
-   VKSCR(CmdBindDescriptorBuffersEXT)(batch->state->cmdbuf, count, infos);
-   VKSCR(CmdBindDescriptorBuffersEXT)(batch->state->reordered_cmdbuf, count, infos);
-   batch->state->dd.db_bound = true;
+   VKSCR(CmdBindDescriptorBuffersEXT)(bs->cmdbuf, count, infos);
+   VKSCR(CmdBindDescriptorBuffersEXT)(bs->reordered_cmdbuf, count, infos);
+   bs->dd.db_bound = true;
 }
 
 /* called on context creation and after flushing an old batch */
 void
-zink_start_batch(struct zink_context *ctx, struct zink_batch *batch)
+zink_start_batch(struct zink_context *ctx)
 {
    struct zink_screen *screen = zink_screen(ctx->base.screen);
-   zink_reset_batch(ctx, batch);
+   zink_reset_batch(ctx);
+   struct zink_batch_state *bs = ctx->bs;
 
-   batch->state->usage.unflushed = true;
+   bs->usage.unflushed = true;
 
    VkCommandBufferBeginInfo cbbi = {0};
    cbbi.sType = VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO;
@@ -543,28 +538,23 @@ zink_start_batch(struct zink_context *ctx, struct zink_batch *batch)
 
    VkResult result;
    VRAM_ALLOC_LOOP(result,
-      VKCTX(BeginCommandBuffer)(batch->state->cmdbuf, &cbbi),
+      VKCTX(BeginCommandBuffer)(bs->cmdbuf, &cbbi),
       if (result != VK_SUCCESS)
          mesa_loge("ZINK: vkBeginCommandBuffer failed (%s)", vk_Result_to_str(result));
    );
    VRAM_ALLOC_LOOP(result,
-      VKCTX(BeginCommandBuffer)(batch->state->reordered_cmdbuf, &cbbi),
+      VKCTX(BeginCommandBuffer)(bs->reordered_cmdbuf, &cbbi),
       if (result != VK_SUCCESS)
          mesa_loge("ZINK: vkBeginCommandBuffer failed (%s)", vk_Result_to_str(result));
    );
    VRAM_ALLOC_LOOP(result,
-      VKCTX(BeginCommandBuffer)(batch->state->unsynchronized_cmdbuf, &cbbi),
+      VKCTX(BeginCommandBuffer)(bs->unsynchronized_cmdbuf, &cbbi),
       if (result != VK_SUCCESS)
          mesa_loge("ZINK: vkBeginCommandBuffer failed (%s)", vk_Result_to_str(result));
    );
 
-   batch->state->fence.completed = false;
-   if (ctx->last_fence) {
-      struct zink_batch_state *last_state = zink_batch_state(ctx->last_fence);
-      batch->last_batch_usage = &last_state->usage;
-   }
+   bs->fence.completed = false;
 
-#ifdef HAVE_RENDERDOC_APP_H
    if (VKCTX(CmdInsertDebugUtilsLabelEXT) && screen->renderdoc_api) {
       VkDebugUtilsLabelEXT capture_label;
       /* Magic fallback which lets us bridge the Wine barrier over to Linux RenderDoc. */
@@ -572,9 +562,9 @@ zink_start_batch(struct zink_context *ctx, struct zink_batch *batch)
       capture_label.pNext = NULL;
       capture_label.pLabelName = "vr-marker,frame_end,type,application";
       memset(capture_label.color, 0, sizeof(capture_label.color));
-      VKCTX(CmdInsertDebugUtilsLabelEXT)(batch->state->unsynchronized_cmdbuf, &capture_label);
-      VKCTX(CmdInsertDebugUtilsLabelEXT)(batch->state->reordered_cmdbuf, &capture_label);
-      VKCTX(CmdInsertDebugUtilsLabelEXT)(batch->state->cmdbuf, &capture_label);
+      VKCTX(CmdInsertDebugUtilsLabelEXT)(bs->unsynchronized_cmdbuf, &capture_label);
+      VKCTX(CmdInsertDebugUtilsLabelEXT)(bs->reordered_cmdbuf, &capture_label);
+      VKCTX(CmdInsertDebugUtilsLabelEXT)(bs->cmdbuf, &capture_label);
    }
 
    unsigned renderdoc_frame = p_atomic_read(&screen->renderdoc_frame);
@@ -583,16 +573,15 @@ zink_start_batch(struct zink_context *ctx, struct zink_batch *batch)
       screen->renderdoc_api->StartFrameCapture(RENDERDOC_DEVICEPOINTER_FROM_VKINSTANCE(screen->instance), NULL);
       screen->renderdoc_capturing = true;
    }
-#endif
 
    /* descriptor buffers must always be bound at the start of a batch */
    if (zink_descriptor_mode == ZINK_DESCRIPTOR_MODE_DB && !(ctx->flags & ZINK_CONTEXT_COPY_ONLY))
       zink_batch_bind_db(ctx);
    /* zero init for unordered blits */
    if (screen->info.have_EXT_attachment_feedback_loop_dynamic_state) {
-      VKCTX(CmdSetAttachmentFeedbackLoopEnableEXT)(ctx->batch.state->cmdbuf, 0);
-      VKCTX(CmdSetAttachmentFeedbackLoopEnableEXT)(ctx->batch.state->reordered_cmdbuf, 0);
-      VKCTX(CmdSetAttachmentFeedbackLoopEnableEXT)(ctx->batch.state->unsynchronized_cmdbuf, 0);
+      VKCTX(CmdSetAttachmentFeedbackLoopEnableEXT)(ctx->bs->cmdbuf, 0);
+      VKCTX(CmdSetAttachmentFeedbackLoopEnableEXT)(ctx->bs->reordered_cmdbuf, 0);
+      VKCTX(CmdSetAttachmentFeedbackLoopEnableEXT)(ctx->bs->unsynchronized_cmdbuf, 0);
    }
 }
 
@@ -615,7 +604,10 @@ post_submit(void *data, void *gdata, int thread_index)
       zink_screen_timeline_wait(screen, bs->fence.batch_id - 2500, OS_TIMEOUT_INFINITE);
    }
    /* this resets the buffer hashlist for the state's next use */
-   memset(&bs->buffer_indices_hashlist, -1, sizeof(bs->buffer_indices_hashlist));
+   if (bs->hashlist_min != UINT16_MAX)
+      /* only reset a min/max region */
+      memset(&bs->buffer_indices_hashlist[bs->hashlist_min], -1, (bs->hashlist_max - bs->hashlist_min + 1) * sizeof(int16_t));
+   bs->hashlist_min = bs->hashlist_max = UINT16_MAX;
 }
 
 typedef enum {
@@ -626,6 +618,8 @@ typedef enum {
    ZINK_SUBMIT_MAX
 } zink_submit;
 
+#define ZINK_MAX_SIGNALS 3
+
 static void
 submit_queue(void *data, void *gdata, int thread_index)
 {
@@ -644,6 +638,8 @@ submit_queue(void *data, void *gdata, int thread_index)
    /* first submit is just for acquire waits since they have a separate array */
    for (unsigned i = 0; i < ARRAY_SIZE(si); i++)
       si[i].sType = VK_STRUCTURE_TYPE_SUBMIT_INFO;
+   if (bs->sparse_semaphore)
+      util_dynarray_append(&ctx->bs->acquires, VkSemaphore, bs->sparse_semaphore);
    si[ZINK_SUBMIT_WAIT_ACQUIRE].waitSemaphoreCount = util_dynarray_num_elements(&bs->acquires, VkSemaphore);
    si[ZINK_SUBMIT_WAIT_ACQUIRE].pWaitSemaphores = bs->acquires.data;
    while (util_dynarray_num_elements(&bs->acquire_flags, VkPipelineStageFlags) < si[ZINK_SUBMIT_WAIT_ACQUIRE].waitSemaphoreCount) {
@@ -679,9 +675,10 @@ submit_queue(void *data, void *gdata, int thread_index)
    unsigned c = 0;
    if (bs->has_unsync)
       cmdbufs[c++] = bs->unsynchronized_cmdbuf;
-   if (bs->has_barriers)
+   if (bs->has_reordered_work)
       cmdbufs[c++] = bs->reordered_cmdbuf;
-   cmdbufs[c++] = bs->cmdbuf;
+   if (bs->has_work)
+      cmdbufs[c++] = bs->cmdbuf;
    si[ZINK_SUBMIT_CMDBUF].pCommandBuffers = cmdbufs;
    si[ZINK_SUBMIT_CMDBUF].commandBufferCount = c;
    /* assorted signal submit from wsi/externals */
@@ -689,12 +686,12 @@ submit_queue(void *data, void *gdata, int thread_index)
    si[ZINK_SUBMIT_CMDBUF].pSignalSemaphores = bs->signal_semaphores.data;
 
    /* then the signal submit with the timeline (fence) semaphore */
-   VkSemaphore signals[3];
+   VkSemaphore signals[ZINK_MAX_SIGNALS];
    si[ZINK_SUBMIT_SIGNAL].signalSemaphoreCount = !!bs->signal_semaphore;
    signals[0] = bs->signal_semaphore;
    si[ZINK_SUBMIT_SIGNAL].pSignalSemaphores = signals;
    VkTimelineSemaphoreSubmitInfo tsi = {0};
-   uint64_t signal_values[2] = {0};
+   uint64_t signal_values[ZINK_MAX_SIGNALS] = {0};
    tsi.sType = VK_STRUCTURE_TYPE_TIMELINE_SEMAPHORE_SUBMIT_INFO;
    si[ZINK_SUBMIT_SIGNAL].pNext = &tsi;
    tsi.pSignalSemaphoreValues = signal_values;
@@ -706,25 +703,30 @@ submit_queue(void *data, void *gdata, int thread_index)
       signals[si[ZINK_SUBMIT_SIGNAL].signalSemaphoreCount++] = bs->present;
    tsi.signalSemaphoreValueCount = si[ZINK_SUBMIT_SIGNAL].signalSemaphoreCount;
 
+   assert(si[ZINK_SUBMIT_SIGNAL].signalSemaphoreCount <= ZINK_MAX_SIGNALS);
+   assert(tsi.signalSemaphoreValueCount <= ZINK_MAX_SIGNALS);
 
    VkResult result;
-   VRAM_ALLOC_LOOP(result,
-      VKSCR(EndCommandBuffer)(bs->cmdbuf),
-      if (result != VK_SUCCESS) {
-         mesa_loge("ZINK: vkEndCommandBuffer failed (%s)", vk_Result_to_str(result));
-         bs->is_device_lost = true;
-         goto end;
-      }
-   );
-   if (bs->has_barriers) {
+   if (bs->has_work) {
+      VRAM_ALLOC_LOOP(result,
+         VKSCR(EndCommandBuffer)(bs->cmdbuf),
+         if (result != VK_SUCCESS) {
+            mesa_loge("ZINK: vkEndCommandBuffer failed (%s)", vk_Result_to_str(result));
+            bs->is_device_lost = true;
+            goto end;
+         }
+      );
+   }
+   if (bs->has_reordered_work) {
       if (bs->unordered_write_access) {
          VkMemoryBarrier mb;
          mb.sType = VK_STRUCTURE_TYPE_MEMORY_BARRIER;
          mb.pNext = NULL;
          mb.srcAccessMask = bs->unordered_write_access;
-         mb.dstAccessMask = 0;
+         mb.dstAccessMask = VK_ACCESS_NONE;
          VKSCR(CmdPipelineBarrier)(bs->reordered_cmdbuf,
-                                   bs->unordered_write_stages, 0,
+                                   bs->unordered_write_stages,
+                                   screen->info.have_KHR_synchronization2 ? VK_PIPELINE_STAGE_NONE : VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT,
                                    0, 1, &mb, 0, NULL, 0, NULL);
       }
       VRAM_ALLOC_LOOP(result,
@@ -772,6 +774,9 @@ submit_queue(void *data, void *gdata, int thread_index)
    }
    _mesa_set_clear(&bs->dmabuf_exports, NULL);
 
+   if (bs->sparse_semaphore)
+      (void)util_dynarray_pop(&ctx->bs->acquires, VkSemaphore);
+
    bs->usage.submit_count++;
 end:
    cnd_broadcast(&bs->usage.flush);
@@ -782,10 +787,10 @@ end:
 
 /* called during flush */
 void
-zink_end_batch(struct zink_context *ctx, struct zink_batch *batch)
+zink_end_batch(struct zink_context *ctx)
 {
    if (!ctx->queries_disabled)
-      zink_suspend_queries(ctx, batch);
+      zink_suspend_queries(ctx);
 
 
    struct zink_screen *screen = zink_screen(ctx->base.screen);
@@ -815,24 +820,24 @@ zink_end_batch(struct zink_context *ctx, struct zink_batch *batch)
          ctx->oom_flush = true;
    }
 
-   bs = batch->state;
-   if (ctx->last_fence)
-      zink_batch_state(ctx->last_fence)->next = bs;
+   bs = ctx->bs;
+   if (ctx->last_batch_state)
+      ctx->last_batch_state->next = bs;
    else {
       assert(!ctx->batch_states);
       ctx->batch_states = bs;
    }
-   ctx->last_fence = &bs->fence;
+   ctx->last_batch_state = bs;
    ctx->batch_states_count++;
-   batch->work_count = 0;
+   ctx->work_count = 0;
 
    /* this is swapchain presentation semaphore handling */
-   if (batch->swapchain) {
-      if (zink_kopper_acquired(batch->swapchain->obj->dt, batch->swapchain->obj->dt_idx) && !batch->swapchain->obj->present) {
-         batch->state->present = zink_kopper_present(screen, batch->swapchain);
-         batch->state->swapchain = batch->swapchain;
+   if (ctx->swapchain) {
+      if (zink_kopper_acquired(ctx->swapchain->obj->dt, ctx->swapchain->obj->dt_idx) && !ctx->swapchain->obj->present) {
+         bs->present = zink_kopper_present(screen, ctx->swapchain);
+         bs->swapchain = ctx->swapchain;
       }
-      batch->swapchain = NULL;
+      ctx->swapchain = NULL;
    }
 
    if (screen->device_lost)
@@ -882,10 +887,14 @@ zink_end_batch(struct zink_context *ctx, struct zink_batch *batch)
       for (; res; res = zink_resource(res->base.b.next)) {
          VkSemaphore sem = zink_create_exportable_semaphore(screen);
          if (sem)
-            util_dynarray_append(&ctx->batch.state->signal_semaphores, VkSemaphore, sem);
+            util_dynarray_append(&ctx->bs->signal_semaphores, VkSemaphore, sem);
       }
+      bs->has_work = true;
    }
 
+   util_dynarray_foreach(&bs->fences, struct zink_tc_fence*, mfence)
+      (*mfence)->deferred_ctx = NULL;
+
    if (screen->threaded_submit) {
       util_queue_add_job(&screen->flush_queue, bs, &bs->flush_completed,
                          submit_queue, post_submit, 0);
@@ -893,23 +902,29 @@ zink_end_batch(struct zink_context *ctx, struct zink_batch *batch)
       submit_queue(bs, NULL, 0);
       post_submit(bs, NULL, 0);
    }
-#ifdef HAVE_RENDERDOC_APP_H
+
    if (!(ctx->flags & ZINK_CONTEXT_COPY_ONLY) && screen->renderdoc_capturing && p_atomic_read(&screen->renderdoc_frame) > screen->renderdoc_capture_end) {
       screen->renderdoc_api->EndFrameCapture(RENDERDOC_DEVICEPOINTER_FROM_VKINSTANCE(screen->instance), NULL);
       screen->renderdoc_capturing = false;
    }
-#endif
+}
+
+ALWAYS_INLINE static void
+batch_hashlist_update(struct zink_batch_state *bs, unsigned hash)
+{
+   bs->hashlist_min = bs->hashlist_min == UINT16_MAX ? hash : MIN2(hash, bs->hashlist_min);
+   bs->hashlist_max = bs->hashlist_max == UINT16_MAX ? hash : MAX2(hash, bs->hashlist_max);
 }
 
 static int
 batch_find_resource(struct zink_batch_state *bs, struct zink_resource_object *obj, struct zink_batch_obj_list *list)
 {
    unsigned hash = obj->bo->unique_id & (BUFFER_HASHLIST_SIZE-1);
-   int i = bs->buffer_indices_hashlist[hash];
+   int buffer_index = bs->buffer_indices_hashlist[hash];
 
    /* not found or found */
-   if (i < 0 || (i < list->num_buffers && list->objs[i] == obj))
-      return i;
+   if (buffer_index < 0 || (buffer_index < list->num_buffers && list->objs[buffer_index] == obj))
+      return buffer_index;
 
    /* Hash collision, look for the BO in the list of list->objs linearly. */
    for (int i = list->num_buffers - 1; i >= 0; i--) {
@@ -924,6 +939,7 @@ batch_find_resource(struct zink_batch_state *bs, struct zink_resource_object *ob
           * will collide here: ^ and here:   ^,
           * meaning that we should get very few collisions in the end. */
          bs->buffer_indices_hashlist[hash] = i & (BUFFER_HASHLIST_SIZE-1);
+         batch_hashlist_update(bs, hash);
          return i;
       }
    }
@@ -931,25 +947,19 @@ batch_find_resource(struct zink_batch_state *bs, struct zink_resource_object *ob
 }
 
 void
-zink_batch_reference_resource_rw(struct zink_batch *batch, struct zink_resource *res, bool write)
+zink_batch_reference_resource_rw(struct zink_context *ctx, struct zink_resource *res, bool write)
 {
    /* if the resource already has usage of any sort set for this batch, */
-   if (!zink_resource_usage_matches(res, batch->state) ||
+   if (!zink_resource_usage_matches(res, ctx->bs) ||
        /* or if it's bound somewhere */
        !zink_resource_has_binds(res))
       /* then it already has a batch ref and doesn't need one here */
-      zink_batch_reference_resource(batch, res);
-   zink_batch_resource_usage_set(batch, res, write, res->obj->is_buffer);
-}
-
-void
-zink_batch_add_wait_semaphore(struct zink_batch *batch, VkSemaphore sem)
-{
-   util_dynarray_append(&batch->state->acquires, VkSemaphore, sem);
+      zink_batch_reference_resource(ctx, res);
+   zink_batch_resource_usage_set(ctx->bs, res, write, res->obj->is_buffer);
 }
 
 static bool
-batch_ptr_add_usage(struct zink_batch *batch, struct set *s, void *ptr)
+batch_ptr_add_usage(struct zink_context *ctx, struct set *s, void *ptr)
 {
    bool found = false;
    _mesa_set_search_or_add(s, ptr, &found);
@@ -958,9 +968,9 @@ batch_ptr_add_usage(struct zink_batch *batch, struct set *s, void *ptr)
 
 /* this is a vague, handwave-y estimate */
 ALWAYS_INLINE static void
-check_oom_flush(struct zink_context *ctx, const struct zink_batch *batch)
+check_oom_flush(struct zink_context *ctx)
 {
-   const VkDeviceSize resource_size = batch->state->resource_size;
+   const VkDeviceSize resource_size = ctx->bs->resource_size;
    if (resource_size >= zink_screen(ctx->base.screen)->clamp_video_mem) {
        ctx->oom_flush = true;
        ctx->oom_stall = true;
@@ -969,31 +979,31 @@ check_oom_flush(struct zink_context *ctx, const struct zink_batch *batch)
 
 /* this adds a ref (batch tracking) */
 void
-zink_batch_reference_resource(struct zink_batch *batch, struct zink_resource *res)
+zink_batch_reference_resource(struct zink_context *ctx, struct zink_resource *res)
 {
-   if (!zink_batch_reference_resource_move(batch, res))
+   if (!zink_batch_reference_resource_move(ctx, res))
       zink_resource_object_reference(NULL, NULL, res->obj);
 }
 
 /* this adds batch usage */
 bool
-zink_batch_reference_resource_move(struct zink_batch *batch, struct zink_resource *res)
+zink_batch_reference_resource_move(struct zink_context *ctx, struct zink_resource *res)
 {
-   struct zink_batch_state *bs = batch->state;
+   struct zink_batch_state *bs = ctx->bs;
 
-   simple_mtx_lock(&batch->ref_lock);
+   simple_mtx_lock(&bs->ref_lock);
    /* swapchains are special */
    if (zink_is_swapchain(res)) {
       struct zink_resource_object **swapchains = bs->swapchain_obj.data;
       unsigned count = util_dynarray_num_elements(&bs->swapchain_obj, struct zink_resource_object*);
       for (unsigned i = 0; i < count; i++) {
          if (swapchains[i] == res->obj) {
-            simple_mtx_unlock(&batch->ref_lock);
+            simple_mtx_unlock(&bs->ref_lock);
             return true;
          }
       }
       util_dynarray_append(&bs->swapchain_obj, struct zink_resource_object*, res->obj);
-      simple_mtx_unlock(&batch->ref_lock);
+      simple_mtx_unlock(&bs->ref_lock);
       return false;
    }
    /* Fast exit for no-op calls.
@@ -1001,7 +1011,7 @@ zink_batch_reference_resource_move(struct zink_batch *batch, struct zink_resourc
     * are outside of the winsys.
     */
    if (res->obj == bs->last_added_obj) {
-      simple_mtx_unlock(&batch->ref_lock);
+      simple_mtx_unlock(&bs->ref_lock);
       return true;
    }
 
@@ -1018,7 +1028,7 @@ zink_batch_reference_resource_move(struct zink_batch *batch, struct zink_resourc
    }
    int idx = batch_find_resource(bs, res->obj, list);
    if (idx >= 0) {
-      simple_mtx_unlock(&batch->ref_lock);
+      simple_mtx_unlock(&bs->ref_lock);
       return true;
    }
 
@@ -1037,6 +1047,7 @@ zink_batch_reference_resource_move(struct zink_batch *batch, struct zink_resourc
    list->objs[idx] = res->obj;
    unsigned hash = bo->unique_id & (BUFFER_HASHLIST_SIZE-1);
    bs->buffer_indices_hashlist[hash] = idx & 0x7fff;
+   batch_hashlist_update(bs, hash);
    bs->last_added_obj = res->obj;
    if (!(res->base.b.flags & PIPE_RESOURCE_FLAG_SPARSE)) {
       bs->resource_size += res->obj->size;
@@ -1051,23 +1062,23 @@ zink_batch_reference_resource_move(struct zink_batch *batch, struct zink_resourc
        *   performing unbind finishes.
        */
    }
-   check_oom_flush(batch->state->ctx, batch);
-   batch->has_work = true;
-   simple_mtx_unlock(&batch->ref_lock);
+   check_oom_flush(bs->ctx);
+   simple_mtx_unlock(&bs->ref_lock);
    return false;
 }
 
 /* this is how programs achieve deferred deletion */
 void
-zink_batch_reference_program(struct zink_batch *batch,
+zink_batch_reference_program(struct zink_context *ctx,
                              struct zink_program *pg)
 {
-   if (zink_batch_usage_matches(pg->batch_uses, batch->state) ||
-       !batch_ptr_add_usage(batch, &batch->state->programs, pg))
+   struct zink_batch_state *bs = ctx->bs;
+   if (zink_batch_usage_matches(pg->batch_uses, bs) ||
+       !batch_ptr_add_usage(ctx, &bs->programs, pg))
       return;
    pipe_reference(NULL, &pg->reference);
-   zink_batch_usage_set(&pg->batch_uses, batch->state);
-   batch->has_work = true;
+   zink_batch_usage_set(&pg->batch_uses, bs);
+   bs->has_work = true;
 }
 
 /* a fast (hopefully) way to check whether a given batch has completed */
@@ -1110,7 +1121,7 @@ batch_usage_wait(struct zink_context *ctx, struct zink_batch_usage *u, bool tryw
    if (!zink_batch_usage_exists(u))
       return;
    if (zink_batch_usage_is_unflushed(u)) {
-      if (likely(u == &ctx->batch.state->usage))
+      if (likely(u == &ctx->bs->usage))
          ctx->base.flush(&ctx->base, NULL, PIPE_FLUSH_HINT_FINISH);
       else { //multi-context
          mtx_lock(&u->mtx);
