@@ -306,7 +306,7 @@ _mesa_hash_table_set_deleted_key(struct hash_table *ht, const void *deleted_key)
 }
 
 static struct hash_entry *
-hash_table_search(struct hash_table *ht, uint32_t hash, const void *key)
+hash_table_search(const struct hash_table *ht, uint32_t hash, const void *key)
 {
    assert(!key_pointer_is_reserved(ht, key));
 
@@ -342,7 +342,7 @@ hash_table_search(struct hash_table *ht, uint32_t hash, const void *key)
  * modified by the user.
  */
 struct hash_entry *
-_mesa_hash_table_search(struct hash_table *ht, const void *key)
+_mesa_hash_table_search(const struct hash_table *ht, const void *key)
 {
    assert(ht->key_hash_function);
    return hash_table_search(ht, ht->key_hash_function(key), key);
@@ -1006,3 +1006,21 @@ _mesa_hash_table_u64_next_entry(struct hash_table_u64 *ht,
       ._entry = next,
    };
 }
+
+/* Updates the data of a u64 hash_table entry inside a
+ * hash_table_u64_foreach() loop
+ */
+void
+_mesa_hash_table_u64_replace(struct hash_table_u64 *ht,
+                             const struct hash_entry_u64 *ent,
+                             void *new_data)
+{
+   if (ent->_entry) {
+      ent->_entry->data = new_data;
+   } else if (ent->key == FREED_KEY_VALUE) {
+      ht->freed_key_data = new_data;
+   } else {
+      assert(ent->key == DELETED_KEY_VALUE);
+      ht->deleted_key_data = new_data;
+   }
+}
